/*
 * Copyright (c) 2020 Siddharth Chandrasekaran <siddharth@embedjournal.com>
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include <zephyr/kernel.h>
#include <zephyr/device.h>
#include <zephyr/drivers/gpio.h>
#include <zephyr/mgmt/osdp.h>

/* The devicetree node identifier for the "led0" alias. */
#define LED0_NODE DT_ALIAS(led0)


static const struct gpio_dt_spec led0 = GPIO_DT_SPEC_GET_OR(LED0_NODE, gpios, {0});

#define SLEEP_TIME_MS                  (20)
#define CNT_PER_SEC                    (1000 / SLEEP_TIME_MS)

int cmd_handler(void *unused, struct osdp_cmd *p)
{
    printk("App received command %d\n", p->id);

    if (p->id == OSDP_CMD_LED) {
        // Temporary LED control
        if (p->led.temporary.control_code == 1) {
            // Turn on the temporary LED
            printk("Turning on temporary LED %d with color %d\n", p->led.led_number, p->led.temporary.on_color);
            gpio_pin_set(led0.port, led0.pin, 1);  // Set GPIO pin to high
            k_msleep(p->led.temporary.on_count);   // Keep it on for the specified time
        } else if (p->led.temporary.control_code == 0) {
            // Turn off the temporary LED
            printk("Turning off temporary LED %d\n", p->led.led_number);
            gpio_pin_set(led0.port, led0.pin, 0);  // Set GPIO pin to low
            k_msleep(p->led.temporary.off_count);  // Keep it off for the specified time
        }

        // Permanent LED control
        if (p->led.permanent.control_code == 1) {
            // Turn on the permanent LED
            printk("Turning on permanent LED %d with color %d\n", p->led.led_number, p->led.permanent.on_color);
            gpio_pin_set(led0.port, led0.pin, 1);  // Set GPIO pin to high
        } else if (p->led.permanent.control_code == 0) {
            // Turn off the permanent LED
            printk("Turning off permanent LED %d\n", p->led.led_number);
            gpio_pin_set(led0.port, led0.pin, 0);  // Set GPIO pin to low
        }

        return 0;
    }

    // Handle other commands here...

    return -1; // Return error for unsupported commands
}


int main(void)
{
    printk("hello osdp_pd\n");
	int ret, led_state;
	uint32_t cnt = 0;

	if (!gpio_is_ready_dt(&led0)) {
		printk("LED0 GPIO port %s is not ready\n", led0.port->name);
		return 0;
	}

	ret = gpio_pin_configure_dt(&led0, GPIO_OUTPUT_ACTIVE);
	if (ret < 0) {
		printk("Failed to configure gpio port %s pin %d\n",
		       led0.port->name, led0.pin);
		return 0;
	}

	osdp_pd_set_command_callback(cmd_handler, NULL);

	led_state = 0;
	while (1) {
		if ((cnt & 0x7f) == 0x7f) {
			/* show a sign of life */
			led_state = !led_state;
		}
		gpio_pin_set(led0.port, led0.pin, led_state);
		k_msleep(SLEEP_TIME_MS);
		cnt++;
	}
	return 0;
}